@@ -2,9 +2,14 @@ package ELF::Writer;
 use Moo 2;
 use Carp;
 use IO::File;
+BEGIN {
+	# Check if we need a wrapper for 'pack'
+	eval "use ELF::Writer::PackWrapper 'pack'"
+		unless eval{ pack('Q<',1) };
+}
 use namespace::clean;
 
-our $VERSION= '0.000_004';
+our $VERSION= '0.01';
 
 # ABSTRACT: Encode elf files with pure-perl
 
@@ -164,10 +169,10 @@ sub elf_header_len {
 		: croak "Don't know structs for elf class $class";
 }
 our @Elf_Header_Pack= (
-	'a4 C C C C C a7 S< S< L< L< L< L< L< S< S< S< S< S< S<', # 32-bit LE
-	'a4 C C C C C a7 S> S> L> L> L> L> L> S> S> S> S> S> S>', # 32-bit BE
-	'a4 C C C C C a7 S< S< L< Q< Q< Q< L< S< S< S< S< S< S<', # 64-bit LE
-	'a4 C C C C C a7 S> S> L> Q> Q> Q> L> S> S> S> S> S> S>', # 64-bit BE
+	'a4 C C C C C a7 v v V V V V V v v v v v v', # 32-bit LE
+	'a4 C C C C C a7 n n N N N N N n n n n n n', # 32-bit BE
+	'a4 C C C C C a7 v v V Q< Q< Q< V v v v v v v', # 64-bit LE
+	'a4 C C C C C a7 n n N Q> Q> Q> N n n n n n n', # 64-bit BE
 );
 sub _elf_header_packstr {
 	my ($self, $encoding)= @_;
@@ -183,10 +188,10 @@ sub segment_header_elem_len {
 }
 # Note! there is also a field swap between 32bit and 64bit
 our @Segment_Header_Pack= (
-	'L< L< L< L< L< L< L< L<',
-	'L> L> L> L> L> L> L> L>',
-	'L< L< Q< Q< Q< Q< Q< Q<',
-	'L> L> Q> Q> Q> Q> Q> Q>',
+	'V V V V V V V V',
+	'N N N N N N N N',
+	'V V Q< Q< Q< Q< Q< Q<',
+	'N N Q> Q> Q> Q> Q> Q>',
 );
 sub _segment_header_packstr {
 	my ($self, $encoding)= @_;
@@ -201,10 +206,10 @@ sub section_header_elem_len {
 		: croak "Don't know structs for elf class $class";
 }
 our @Section_Header_Pack= (
-	'L< L< L< L< L< L< L< L< L< L<',
-	'L> L> L> L> L> L> L> L> L> L>',
-	'L< L< Q< Q< Q< Q< L< L< Q< Q<',
-	'L> L> Q> Q> Q> Q> L> L> Q> Q>',
+	'V V V V V V V V V V',
+	'N N N N N N N N N N',
+	'V V Q< Q< Q< Q< V V Q< Q<',
+	'N N Q> Q> Q> Q> N N Q> Q>',
 );
 sub _section_header_packstr {
 	my ($self, $encoding)= @_;
@@ -345,7 +350,7 @@ sub serialize {
 		# Otherwise we just create a dummy to use below.
 		if (!defined $segment_table) {
 			$segment_table= ELF::Writer::Segment->new(
-				align => 8,
+				align => ($self->class == 2? 8 : 4),
 				filesize => length($segment_table_data),
 				data => $segment_table_data,
 			);
@@ -360,7 +365,7 @@ sub serialize {
 			for @sections;
 		
 		$section_table= ELF::Writer::Segment->new(
-			align => 8,
+			align => ($self->class == 2? 8 : 4),
 			filesize => length($section_table_data),
 			data => $section_table_data,
 		);
@@ -540,7 +545,7 @@ sub _apply_segment_defaults {
 	}
 }
 
-# Load last so make sure data is initialized
+# Loaded last, to make sure all data in this module is initialized
 require ELF::Writer::Segment;
 require ELF::Writer::Section;
 
@@ -558,7 +563,7 @@ ELF::Writer - Encode elf files with pure-perl
 
 =head1 VERSION
 
-version 0.000_004
+version 0.01
 
 =head1 SYNOPSIS
 
@@ -573,10 +578,9 @@ version 0.000_004
   $elf->write_file($binary_name);
   
   # Example above wastes almost 4K to align the first segment.
-  # We can overlap the first segment with the elf header, using this trick:
-  # http://www.muppetlabs.com/~breadbox/software/tiny/teensy.html
-  # Note that this moves our machine code, which can affect RIP-relative
-  # addressing.
+  # We can overlap the first segment with the elf header, so that the entire
+  # file gets paged into RAM, but then the entry point needs adjusted by the
+  # size of the ELF headers.
   
   my $prog_offset= $elf->elf_header_len + $elf->segment_header_elem_len;
   $elf->segments->[0]->offset(0);
@@ -693,9 +697,9 @@ Handy alias for C<$#{ $elf->sections }>
 
 Handy alias for C<@{ $elf->sections }>
 
-=head2 section_name_string_table_index
+=head2 section_name_string_table_idx
 
-Insex into the section array of a string-table section where the names of
+Index into the section array of a string-table section where the names of
 the sections are stored.
 
 =head1 METHODS
@@ -711,6 +715,15 @@ attributes are missing.
 
 Convenience method for writing to a file.  Writes with mode 0755 by default.
 
+=head1 SEE ALSO
+
+Brian Raiter has a nice write-up of how to hack around on ELF files, which
+I found very educational:
+
+http://www.muppetlabs.com/~breadbox/software/tiny/teensy.html
+
+=cut
+
 =head1 AUTHOR
 
 Michael Conrad <mike@nrdvana.net>
